#include "csapp.h"

/* Run a given program ITERS times, collecting up to
   VARIANTS different outputs, each of which is up
   to BUFFER_MAX bytes long. */

#define VARIANTS 30
#define BUFFER_MAX 256
#define ITERS 100000

int main(int argc, char **argv) {
  char buffer[VARIANTS][BUFFER_MAX+1];
  char *next_argv[2] = { argv[1], NULL};
  int num_variants = 0, k;

  if (argc != 2) {
    fprintf(stderr, "%s: provide a program to run\n", argv[0]);
    return 1;
  }

  for (k = 0; k < ITERS; k++) {
    int fds[2], got = 0, n, i;
    pid_t pid;

    Pipe(fds);
    
    pid = Fork();

    if (pid == 0) {
      Dup2(fds[1], 1);
      Close(fds[1]);
      Close(fds[0]);
      Execve(argv[1], next_argv, environ);
    }

    Close(fds[1]);
    while (1) {
      n = Read(fds[0], buffer[num_variants] + got, BUFFER_MAX-got);
      if (!n)
	break;
      got += n;
      if (got >= BUFFER_MAX)
	break;
    }
    Close(fds[0]);
    buffer[num_variants][got] = 0;

    Waitpid(pid, NULL, 0);

    for (i = 0; i < num_variants; i++) {
      if (!strcmp(buffer[i], buffer[num_variants]))
	break;
    }
    if (i == num_variants) {
      printf("GOT %d:\n%s\n", num_variants+1, buffer[num_variants]);
      num_variants++;
      if (num_variants == VARIANTS)
	break;
    }
  }

  return 0;
}
